`timescale 1ns/1ps

module testbench;
parameter WORD = 1;
    parameter HOR = 28;

    parameter ADDR_W = $clog2(HOR);
    parameter Q = 3 ;    // number of fractional bits

logic clock;
logic reset;
logic data_valid_in;
logic [WORD-1:0] pixel_data_in;
logic [ADDR_W-1:0] hcount_in;
logic [$clog2(HOR)-1:0] vcount_in;
logic signed [7:0] weight_in;
logic [2:0] weight_add;

logic signed [11:0] mac_out;
logic [WORD-1:0] mem0_out [0:HOR-1];
logic [WORD-1:0] mem1_out [0:HOR-1];
logic [WORD-1:0] mem2_out [0:HOR-1];
logic [WORD-1:0] mem3_out [0:HOR-1];
logic [WORD-1:0] top_line_out [0:2];
logic [WORD-1:0] mid_line_out [0:2];
logic [WORD-1:0] bot_line_out [0:2];

convolver dut(
    .clock(clock),
    .reset(reset),
    .data_valid_in(data_valid_in),
    .pixel_data_in(pixel_data_in),
    .hcount_in(hcount_in),
    .vcount_in(vcount_in),
    .weight_in(weight_in),
    .weight_add(weight_add),
    .mac_out(mac_out),

    .mem0_out(mem0_out),
    .mem1_out(mem1_out),
    .mem2_out(mem2_out),
    .mem3_out(mem3_out),
    .top_line_out(top_line_out),
    .mid_line_out(mid_line_out),
    .bot_line_out(bot_line_out)
    );

always begin
    #500;
    clock = ~clock;
end
logic [60:0] counter;
always_ff @(negedge clock) begin
    if(reset) begin
        counter <= 0;
    end
    else begin
        counter <= counter + 1;
    end
end

initial begin
    clock = 0;
    reset = 0;
    data_valid_in = 0;
    pixel_data_in = 0;
    hcount_in = 0;
    vcount_in = 0;
    weight_add = 0;
    weight_in = 0;


    @(negedge clock);

    @(negedge clock);
    reset =1;
    @(negedge clock);
    reset =0;
    @(negedge clock);
    @(negedge clock);
    data_valid_in = 1;
    for (integer i=0; i<28; i++) begin
        hcount_in = i;
        pixel_data_in = 1;
        if(i < 9) begin
            weight_add = i;
            weight_in = i;
        end
        @(negedge clock);
    end
    $display("CLOCK CYCLE: %0d", counter);
    $display("MEM 0");
    for (integer i=0; i<28; i++) begin
        $display("row:%0d =%0d", i, mem0_out[i]);
    end
    /////////////////////////////////////////////////////////////////////////
    for (integer i=0; i<28; i++) begin
        hcount_in = i;
        pixel_data_in = 0;
        @(negedge clock);
    end
    $display("CLOCK CYCLE: %0d", counter);
    $display("MEM 1");
    for (integer i=0; i<28; i++) begin
        $display("row:%0d =%0d", i, mem1_out[i]);
    end
    //////////////////////////////////////////////////////
    for (integer i=0; i<28; i++) begin
        hcount_in = i;
        pixel_data_in = 1;
        @(negedge clock);
    end
    $display("CLOCK CYCLE: %0d", counter);
    $display("MEM 2");
    for (integer i=0; i<28; i++) begin
        $display("row:%0d =%0d", i, mem2_out[i]);
    end
    ///////////////////////////////////////////////////////
    for (integer i=0; i<28; i++) begin
        $display("CLOCK CYCLE: %0d", counter);
        $display("TOP_LINE: %0d, %0d, %0d",top_line_out[0],top_line_out[1], top_line_out[2]);
        $display("MID_LINE: %0d, %0d, %0d",mid_line_out[0],mid_line_out[1], mid_line_out[2]);
        $display("BOT_LINE: %0d, %0d, %0d",bot_line_out[0],bot_line_out[1], bot_line_out[2]);
        $display("MULTIPLY ACCUMULATE RESULTS: %0d", mac_out);
        hcount_in = i;
        pixel_data_in = 0;
        @(negedge clock);
    end
    $display("CLOCK CYCLE: %0d", counter);
    $display("MEM 3");
    for (integer i=0; i<28; i++) begin
        $display("row:%0d =%0d", i, mem3_out[i]);
    end
    for (integer i=0; i<28; i++) begin
        $display("CLOCK CYCLE: %0d", counter);
        $display("TOP_LINE: %0d, %0d, %0d",top_line_out[0],top_line_out[1], top_line_out[2]);
        $display("MID_LINE: %0d, %0d, %0d",mid_line_out[0],mid_line_out[1], mid_line_out[2]);
        $display("BOT_LINE: %0d, %0d, %0d",bot_line_out[0],bot_line_out[1], bot_line_out[2]);
        $display("MULTIPLY ACCUMULATE RESULTS: %0d", mac_out);
        hcount_in = i;
        pixel_data_in = 0;
        @(negedge clock);
    end
    $display ( "-----------------------------------------------------------------------------------------------------------------------------");
    for (integer i=0; i<28; i++) begin
        $display("CLOCK CYCLE: %0d", counter);
        $display("TOP_LINE: %0d, %0d, %0d",top_line_out[0],top_line_out[1], top_line_out[2]);
        $display("MID_LINE: %0d, %0d, %0d",mid_line_out[0],mid_line_out[1], mid_line_out[2]);
        $display("BOT_LINE: %0d, %0d, %0d",bot_line_out[0],bot_line_out[1], bot_line_out[2]);
        $display("MULTIPLY ACCUMULATE RESULTS: %0d", mac_out);
        hcount_in = i;
        pixel_data_in = 1;
        @(negedge clock);
    end
    ////////////////////////////////////////////////////////////

    $finish;
end

endmodule