// INITIAL STATE//
//////////////////////
//     image        //
//////////////////////
//     row 0        //  out2
//////////////////////
//     row 1        //  out1
//////////////////////
//     row 2        //  out0
//////////////////////
//     row 3        //  write line
//////////////////////
module Line_Buff#(
    parameter WORD = 8,
    parameter HOR = 28,

    parameter ADDR_W = $clog2(HOR)
)(
    input logic clock,
    input logic reset,
    input logic data_valid_in,
    input logic signed [WORD-1:0] pixel_data_in,
    input logic [ADDR_W-1:0] hcount_in,

    output logic signed [WORD-1:0] line_buff_out [0:2],
    output logic [ADDR_W-1:0] hcount_out,
    output logic signed [WORD-1:0] mem0_out [0:HOR-1],
    output logic signed [WORD-1:0] mem1_out [0:HOR-1],
    output logic signed [WORD-1:0] mem2_out [0:HOR-1],
    output logic signed [WORD-1:0] mem3_out [0:HOR-1]


);

logic signed [WORD-1:0] mem0 [0:HOR-1];
logic signed [WORD-1:0] mem1 [0:HOR-1];
logic signed [WORD-1:0] mem2 [0:HOR-1];
logic signed [WORD-1:0] mem3 [0:HOR-1];

assign mem0_out = mem0;
assign mem1_out = mem1;
assign mem2_out = mem2;
assign mem3_out = mem3;

logic[1:0] current_write;
logic signed [WORD-1:0] out0,out1,out2;
logic valid;

always_ff @(posedge clock) begin
    if (reset) begin
        out0 <= 0;
        out1 <= 0;
        out2 <= 0;
        valid <= 0;
        hcount_out <= 0;
        current_write <= 0;
        for (integer i = 0; i < HOR; i++) begin
            mem0[i] <= 0;
            mem1[i] <= 0;
            mem2[i] <= 0;
            mem3[i] <= 0;
        end
    end
    else begin
        valid <= data_valid_in;
        hcount_out <= hcount_in;
        if (data_valid_in) begin
            /// write incoming pixel into current memory
            case (current_write)
                2'd0 : mem0[hcount_in] <= pixel_data_in;
                2'd1 : mem1[hcount_in] <= pixel_data_in;
                2'd2 : mem2[hcount_in] <= pixel_data_in;
                2'd3 : mem3[hcount_in] <= pixel_data_in;
                default: mem0[hcount_in] <= 0;
            endcase
        end
            ///// reading values from the array
            unique case (current_write +2'd3)
                2'd0 : out0 <= mem0[hcount_in];
                2'd1 : out0 <= mem1[hcount_in];
                2'd2 : out0 <= mem2[hcount_in];
                2'd3 : out0 <= mem3[hcount_in];
                default: out0 <= 0;
            endcase

            unique case (current_write +2'd2)
                2'd0 : out1 <= mem0[hcount_in];
                2'd1 : out1 <= mem1[hcount_in];
                2'd2 : out1 <= mem2[hcount_in];
                2'd3 : out1 <= mem3[hcount_in];
                default: out1 <= 0;
            endcase

            unique case (current_write +2'd1)
                2'd0 : out2 <= mem0[hcount_in];
                2'd1 : out2 <= mem1[hcount_in];
                2'd2 : out2 <= mem2[hcount_in];
                2'd3 : out2 <= mem3[hcount_in];
                default: out2 <= 0;
            endcase

            if ((hcount_in == 27)) begin
            current_write <= current_write + 1'd1;
        end
        end

    end


/*always_ff @(posedge clock) begin
    if (reset) begin
        current_write <=0;
    end
    if ((hcount_in == 27)) begin
            current_write <= current_write + 1'd1;
        end
end*/

  /// outputs
    assign line_buff_out[0] = out0;  // pixel @(h, v-3)
    assign line_buff_out[1] = out1;  // pixel @(h, v-2)
    assign line_buff_out[2] = out2;  // pixel @(h, v-1)
endmodule


/////////////////////////////////////////////////////////////////
//out2->// top_line[0]*W[0]  top_line[1]*W[1]  top_line[2]*W[2]//
/////////////////////////////////////////////////////////////////
//out1->// mid_line[0]*W[3]  mid_line[1]*W[4]  mid_line[2]*W[5]//
/////////////////////////////////////////////////////////////////
//out0->// bot_line[0]*W[6]  bot_line[1]*W[7]  bot_line[2]*W[8]//
/////////////////////////////////////////////////////////////////

module convolver#(
    parameter WORD = 1,
    parameter HOR = 28,

    parameter ADDR_W = $clog2(HOR),
    parameter Q = 3     // number of fractional bits
) (
    input logic clock,
    input logic reset,
    input logic data_valid_in,
    input logic [WORD-1:0] pixel_data_in,
    input logic [ADDR_W-1:0] hcount_in,
    input logic signed [7:0] weight_in,
    input logic weight_en,
    input logic pool_valid,
    input logic signed [7:0] bias,
    //input logic [3:0] weight_add,

    output logic signed [31:0] mac_out,
    output logic [ADDR_W-1:0] hcount,
    output logic signed [7:0] mem0_out [0:HOR-1],
    output logic signed [7:0] mem1_out [0:HOR-1],
    output logic signed [7:0] mem2_out [0:HOR-1],
    output logic signed [7:0] mem3_out [0:HOR-1],
    output logic signed [7:0] top_line_out [0:2],
    output logic signed [7:0] mid_line_out [0:2],
    output logic signed [7:0] bot_line_out [0:2]
);
///////internal signals///////

logic signed [7:0] line_buff_out [0:2];
logic [ADDR_W-1:0] hcount_out;
logic signed[7:0] pixel;

assign pixel = (pixel_data_in) ? 127: -128;

/////////////////////////////////

Line_Buff lb(
    .clock(clock),
    .reset(reset),
    .data_valid_in(data_valid_in),
    .pixel_data_in(pixel),
    .hcount_in(hcount_in),

    .line_buff_out(line_buff_out),
    .hcount_out(hcount_out),
    .mem0_out(mem0_out),
    .mem1_out(mem1_out),
    .mem2_out(mem2_out),
    .mem3_out(mem3_out)
);

logic signed [7:0] top_line [0:2];
logic signed [7:0] mid_line [0:2];
logic signed [7:0] bot_line [0:2];

logic signed [31:0] part_mult0, part_mult1, part_mult2;
logic signed [31:0] part_mult3, part_mult4, part_mult5;
logic signed [31:0] part_mult6, part_mult7, part_mult8;

logic signed [31:0] partial1_add1, partial1_add2, partial1_add3, partial1_add4;
logic signed [31:0] partial2_add1, partial2_add2;
logic signed [31:0] partial3_add1;
logic signed [31:0] acc;
logic signed [7:0] weight [0:8];
logic [3:0] weight_add;



always_ff @(posedge clock) begin
    if(reset) begin
        for(integer i =0; i<9; i++) begin
            weight[i] <=0;
        end
        weight_add <= 0;
    end
    else if (weight_en) begin
        weight[weight_add] <= weight_in;
        weight_add <= (weight_add +1) % 9;
    end
end
/*always_comb begin
$display("Weight_Add: %0d, Weight_in:%0d, Weight_en: %0d", weight_add, weight_in, weight_en);
end*/

assign top_line_out =top_line;
assign mid_line_out = mid_line;
assign bot_line_out = bot_line;
always_ff @(posedge clock) begin
    if (reset) begin
        top_line[0] <= 0;
        top_line[1] <= 0;
        top_line[2] <= 0;
        mid_line[0] <= 0;
        mid_line[1] <= 0;
        mid_line[2] <= 0;
        bot_line[0] <= 0;
        bot_line[1] <= 0;
        bot_line[2] <= 0;
    end
    else begin
        top_line[2] <= top_line[1];
        top_line[1] <= top_line[0];
        top_line[0] <= line_buff_out[2];

        mid_line[2] <= mid_line[1];
        mid_line[1] <= mid_line[0];
        mid_line[0] <= line_buff_out[1];

        bot_line[2] <= bot_line[1];
        bot_line[1] <= bot_line[0];
        bot_line[0] <= line_buff_out[0];

        hcount <= hcount_out;
    end
end

always_ff @(posedge clock) begin
    if(reset) begin
        part_mult0 <= 0;
        part_mult1 <= 0;
        part_mult2 <= 0;
        part_mult3 <= 0;
        part_mult4 <= 0;
        part_mult5 <= 0;
        part_mult6 <= 0;
        part_mult7 <= 0;
        part_mult8 <= 0;

        partial1_add1 <= 0;
        partial1_add2 <= 0;
        partial1_add3 <= 0;
        partial1_add4 <= 0;

        partial2_add1 <= 0;
        partial2_add2 <= 0;

        partial3_add1 <= 0;

        acc <= 0;
    end
    else begin
        part_mult0 <= $signed({{24{top_line[0][7]}}, top_line[0]}) * $signed({{24{weight[0][7]}}, weight[0]});
        part_mult1 <= $signed({{24{top_line[1][7]}}, top_line[1]}) * $signed({{24{weight[1][7]}}, weight[1]});
        part_mult2 <= $signed({{24{top_line[2][7]}}, top_line[2]}) * $signed({{24{weight[2][7]}}, weight[2]});

        part_mult3 <= $signed({{24{mid_line[0][7]}}, mid_line[0]}) * $signed({{24{weight[3][7]}}, weight[3]});
        part_mult4 <= $signed({{24{mid_line[1][7]}}, mid_line[1]}) * $signed({{24{weight[4][7]}}, weight[4]});
        part_mult5 <= $signed({{24{mid_line[2][7]}}, mid_line[2]}) * $signed({{24{weight[5][7]}}, weight[5]});

        part_mult6 <= $signed({{24{bot_line[0][7]}}, bot_line[0]}) * $signed({{24{weight[6][7]}}, weight[6]});
        part_mult7 <= $signed({{24{bot_line[1][7]}}, bot_line[1]}) * $signed({{24{weight[7][7]}}, weight[7]});
        part_mult8 <= $signed({{24{bot_line[2][7]}}, bot_line[2]}) * $signed({{24{weight[8][7]}}, weight[8]});

        partial1_add1 <= part_mult0 + part_mult1;
        partial1_add2 <= part_mult2 + part_mult3;
        partial1_add3 <= part_mult4 + part_mult5;
        partial1_add4 <= part_mult6 + part_mult7;

        partial2_add1 <= partial1_add1 + partial1_add2;
        partial2_add2 <= partial1_add3 + partial1_add4;

        partial3_add1 <= partial2_add1 + partial2_add2;

        acc <= partial3_add1 + part_mult8 + $signed({{24{bias[7]}}, bias});
    end
end

/*
integer fd;
assign fd = $fopen("internals_conv.txt", "w");
always_ff @(posedge clock) begin
    $fdisplay(fd,"Inputs: %0d, %0d, %0d, %0d, %0d, %0d, %0d, %0d, %0d",
    top_line[0],top_line[1],top_line[2],mid_line[0], mid_line[1],mid_line[2], bot_line[0],bot_line[1], bot_line[2]);
    $fdisplay(fd,"Multiplications: %0d, %0d, %0d, %0d, %0d, %0d, %0d, %0d, %0d", 
    part_mult0,part_mult1,part_mult2,part_mult3,part_mult4,part_mult5,part_mult6,part_mult7,part_mult8);
    $fdisplay(fd," First stage of adder tree: %0d, %0d, %0d, %0d",partial1_add1,partial1_add2,partial1_add3,partial1_add4);
    $fdisplay(fd,"Second stage of adder tree: %0d, %0d", partial2_add1, partial2_add2);
    $fdisplay(fd,"Third Stage of adder tree: %0d", partial3_add1);
    $fdisplay(fd, " Final acc value: %0d", acc);
    $fdisplay(fd,"                ");
end
*/

// Add ReLU activation
assign mac_out = (acc > 0) ? acc-128 : -128;

endmodule