// Parameterized Dual-Port Synchronous RAM for Data Memory

module dense_out_mem #(
    parameter DATA_WIDTH = 32,
    parameter MEM_DEPTH  = 10,
    parameter ADDR_WIDTH = $clog2(MEM_DEPTH)
) (
    input logic                  clk,

    input logic                  we_a,
    input logic [ADDR_WIDTH-1:0] addr_a,
    input logic [DATA_WIDTH-1:0] data_in_a,
    output logic [DATA_WIDTH-1:0] data_out_a,

    input logic                  we_b,
    input logic [ADDR_WIDTH-1:0] addr_b,
    input logic [DATA_WIDTH-1:0] data_in_b,
    output logic [DATA_WIDTH-1:0] data_out_b
);

    logic [DATA_WIDTH-1:0] mem_array [MEM_DEPTH-1:0];

    always_ff @(posedge clk) begin
        if (we_a) begin

            mem_array[addr_a] <= data_in_a;
        end

        data_out_a <= mem_array[addr_a];

        if (we_b) begin
            mem_array[addr_b] <= data_in_b;
        end
        data_out_b <= mem_array[addr_b];

    end

endmodule