// INITIAL STATE//
//////////////////////
//     image        // 
//////////////////////
//     row 0        //  out2
//////////////////////
//     row 1        //  out1
//////////////////////
//     row 2        //  out0
//////////////////////
//     row 3        //  write line
//////////////////////
module Line_Buff1#(
    parameter WORD = 32,
    parameter HOR = 26,

    parameter ADDR_W = $clog2(HOR) 
)(
    input logic clock,
    input logic reset,
    input logic data_valid_in,
    input logic signed [WORD-1:0] pixel_data_in,
    input logic [ADDR_W-1:0] hcount_in,

    output logic signed [WORD-1:0] line_buff_out [0:2],
    output logic [ADDR_W-1:0] hcount_out,
    output logic signed [WORD-1:0] mem0_out [0:HOR-1],
    output logic signed [WORD-1:0] mem1_out [0:HOR-1],
    output logic signed [WORD-1:0] mem2_out [0:HOR-1],
    output logic signed [WORD-1:0] mem3_out [0:HOR-1]


);

logic signed [WORD-1:0] mem0 [0:HOR-1];
logic signed [WORD-1:0] mem1 [0:HOR-1];
logic signed [WORD-1:0] mem2 [0:HOR-1];
logic signed [WORD-1:0] mem3 [0:HOR-1];

assign mem0_out = mem0;
assign mem1_out = mem1;
assign mem2_out = mem2;
assign mem3_out = mem3;

logic[1:0] current_write;
logic signed [WORD-1:0] out0,out1,out2;
logic valid;

always_ff @(posedge clock) begin
    if (reset) begin
        out0 <= 0;
        out1 <= 0;
        out2 <= 0;
        valid <= 0;
        hcount_out <= 0;
        for (integer i = 0; i < HOR; i++) begin
            mem0[i] <= 0;
            mem1[i] <= 0;
            mem2[i] <= 0;
            mem3[i] <= 0;
        end
    end
    else begin
        valid <= data_valid_in;
        hcount_out <= hcount_in;
        if (data_valid_in) begin
            /// write incoming pixel into current memory
            case (current_write)
                2'd0 : mem0[hcount_in] <= pixel_data_in; 
                2'd1 : mem1[hcount_in] <= pixel_data_in;
                2'd2 : mem2[hcount_in] <= pixel_data_in;
                2'd3 : mem3[hcount_in] <= pixel_data_in;
                default: mem0[hcount_in] <= 0;
            endcase

            ///// reading values from the array
            unique case (current_write +2'd3)
                2'd0 : out0 <= mem0[hcount_in];
                2'd1 : out0 <= mem1[hcount_in];
                2'd2 : out0 <= mem2[hcount_in];
                2'd3 : out0 <= mem3[hcount_in];
                default: out0 <= 0;
            endcase

            unique case (current_write +2'd2)
                2'd0 : out1 <= mem0[hcount_in];
                2'd1 : out1 <= mem1[hcount_in];
                2'd2 : out1 <= mem2[hcount_in];
                2'd3 : out1 <= mem3[hcount_in];
                default: out1 <= 0;
            endcase

            unique case (current_write +2'd1)
                2'd0 : out2 <= mem0[hcount_in];
                2'd1 : out2 <= mem1[hcount_in];
                2'd2 : out2 <= mem2[hcount_in];
                2'd3 : out2 <= mem3[hcount_in];
                default: out2 <= 0;
            endcase
        end
        
    end
end

always_ff @(posedge clock) begin
    if (reset) begin
        current_write <=0;
    end
    if ((hcount_in == 25)) begin
            current_write <= current_write + 1'd1;
        end
end


  /// outputs
    assign line_buff_out[0] = out0;  // pixel @(h, v-3)
    assign line_buff_out[1] = out1;  // pixel @(h, v-2)
    assign line_buff_out[2] = out2;  // pixel @(h, v-1)
endmodule

/////////////////////////////////////////////////////////////////
//out2->// top_line[0]*W[6]  top_line[1]*W[7]  top_line[2]*W[8]//
/////////////////////////////////////////////////////////////////
//out1->// mid_line[0]*W[3]  mid_line[1]*W[4]  mid_line[2]*W[5]//
/////////////////////////////////////////////////////////////////


module pooling#(
    parameter WORD = 32,
    parameter HOR = 26,

    parameter ADDR_W = $clog2(HOR),
    parameter Q = 3     // number of fractional bits
) (
    input logic clock,
    input logic reset,
    input logic data_valid_in,
    input logic signed [WORD-1:0] data_in,
    input logic [ADDR_W-1:0] hcount_in,

    output logic signed [7:0] pooled_out,
    output logic signed [WORD-1:0] mem0_out [0:HOR-1],
    output logic signed [WORD-1:0] mem1_out [0:HOR-1],
    output logic signed [WORD-1:0] mem2_out [0:HOR-1],
    output logic signed [WORD-1:0] mem3_out [0:HOR-1],
    output logic signed [WORD-1:0] top_line_out [0:1],
    output logic signed [WORD-1:0] mid_line_out [0:1]
);
///////internal signals///////

logic signed [WORD-1:0] line_buff_out [0:2];

/////////////////////////////////

Line_Buff1 lb(
    .clock(clock),
    .reset(reset),
    .data_valid_in(data_valid_in),
    .pixel_data_in(data_in),
    .hcount_in(hcount_in),

    .line_buff_out(line_buff_out),
    .hcount_out(),
    .mem0_out(mem0_out),
    .mem1_out(mem1_out),
    .mem2_out(mem2_out),
    .mem3_out(mem3_out)
);

logic signed [WORD-1:0] top_line [0:1];
logic signed [WORD-1:0] mid_line [0:1];

logic signed [WORD-1:0] comparison1;
logic signed [WORD-1:0] comparison2;
logic signed [WORD-1:0] comparison3;

assign top_line_out = top_line;
assign mid_line_out = mid_line;

always_ff @(posedge clock) begin
    if (reset) begin
        top_line[0] <= 0;
        top_line[1] <= 0;
        mid_line[0] <= 0;
        mid_line[1] <= 0;
    end
    else begin
        top_line[1] <= top_line[0];
        top_line[0] <= line_buff_out[2];

        mid_line[1] <= mid_line[0];
        mid_line[0] <= line_buff_out[1];

    end
end

always_ff @(posedge clock) begin
    if(reset) begin
        comparison1 <= 0;
        comparison2 <= 0;
        comparison3 <= 0;
    end
    else begin
        comparison1 <= (mid_line[1] > mid_line[0]) ? mid_line[1] : mid_line[0];
        comparison2 <= (top_line[1] > top_line[0]) ? top_line[1] : top_line[0];
        comparison3 <= (comparison1 > comparison2) ? (((11*comparison1) >>> 10)) : (((11*comparison2) >>> 10));
        
    end
end

assign pooled_out =comparison3;

endmodule

