module top (
    input logic         clk,
    input logic 	    reset,
    input logic [31:0]  writedata,
    input logic 	    write,
    input               chipselect,
    input logic [3:0]   address,

    output logic [31:0] readdata
);
logic [31:0] dense_data;
logic dense_done;
logic image_data;
logic [9:0] image_address;
logic load_done;

always_comb begin
    readdata = 32'b0;

    if (reset) begin
        readdata = 0;
    end
    else if (chipselect && ~write) begin
        case (address)
            4'h0 : readdata = dense_data;
            4'h1 : readdata = dense_data;
            4'h2 : readdata = dense_data;
            4'h3 : readdata = dense_data;
            4'h4 : readdata = dense_data;
            4'h5 : readdata = dense_data;
            4'h6 : readdata = dense_data;
            4'h7 : readdata = dense_data;
            4'h8 : readdata = dense_data;
            4'h9 : readdata = dense_data;
            4'hF : readdata = {31'b0, dense_done};
            default: readdata = dense_data;
        endcase
    end
end
always_ff @(posedge clk) begin
    if(reset) begin
        image_data <= 0;
        image_address <= 0;
        load_done <= 0;
    end
    else if (chipselect && write) begin
        image_data <= writedata[0];
        image_address <= writedata[10:1];
        load_done <= writedata[11];
    end
end

control control1(
    .clock(clk),
    .reset(reset),
    .image_load_done(load_done),
    .image_data(image_data),
    .image_write_en(chipselect && write),
    .dense_done(dense_done),
    .dense_address(address),
    .dense_output(dense_data)
);

endmodule
