import numpy as np
import tensorflow as tf
from tensorflow import keras
import pathlib

# --- 1) Data loading & preprocessing ---
num_classes = 10
input_shape = (28, 28, 1)
(x_train, y_train), (x_test, y_test) = keras.datasets.mnist.load_data()
# binarize to 0/1 and cast to float32
x_train = np.where(x_train > 127, 1, 0).astype(np.float32)
x_test  = np.where(x_test  > 127, 1, 0).astype(np.float32)
# add channel dim
x_train = np.expand_dims(x_train, -1)
x_test  = np.expand_dims(x_test, -1)
print(y_test[0:1000])
# one-hot labels
y_train = keras.utils.to_categorical(y_train, num_classes)
y_test  = keras.utils.to_categorical(y_test, num_classes)
print(f"x_train: {x_train.shape}, x_test: {x_test.shape}\n")

out_dir = pathlib.Path("quartus_weights_mif")
out_dir.mkdir(exist_ok=True)

fname = out_dir / f"x_train.mem"
with open(fname, "w") as f:
	for image in x_train[:100]:
		for row in image:
			for col in row:
				for pixel in col:
					f.write(f"{int(pixel)}\n")

print("Train complete")

fname = out_dir / f"x_test.mem"
with open(fname, "w") as f:
	for image in x_test[:100]:
		for row in image:
			for col in row:
				for pixel in col:
					f.write(f"{int(pixel)}\n")

print("Test complete")