import numpy as np
import tensorflow as tf
import tensorflow.keras as keras
import pathlib

num_classes = 10
input_shape = (28, 28, 1)
(x_train, y_train), (x_test, y_test) = keras.datasets.mnist.load_data()
# binarize to 0/1 and cast to float32
x_train = np.where(x_train > 127, 127, -128).astype(np.float32)
x_test  = np.where(x_test  > 127, 127, -128).astype(np.float32)
# add channel dim
x_train = np.expand_dims(x_train, -1)
x_test  = np.expand_dims(x_test, -1)

#x_train = np.where(x_train == 1, 127, -128).astype(np.int8)
#x_test = np.where(x_train == 1, 127, -128).astype(np.int8)
# === EXTRA: Preview first‑convolution feature maps ==========================
# Grab the first test image, quantize it with the same scale/zero‑point as the
# model input, run one inference, and then pull out the tensor that contains
# the output of the *first* convolution (index 8 in this model).
print("\n--- First‑convolution output (de‑quantized) for test image 0 ---")
feat_interp = tf.lite.Interpreter(
    model_path="mnist_int8.tflite",
    experimental_preserve_all_tensors=True   # <-- key flag
)
feat_interp.allocate_tensors()

# 2.  Quantise one test image exactly like the model expects
in_info          = feat_interp.get_input_details()[0]
in_scale, in_zp  = in_info["quantization"]
img0_q = np.clip(
    np.round(x_test[0:1] / in_scale),
    -128, 127
).astype(np.int8)
feat_interp.set_tensor(in_info["index"], img0_q)

out_dir = pathlib.Path("first_layer_output")
out_dir.mkdir(exist_ok=True)

fname = out_dir / f"adjusted_input"
with open(fname, "w") as f:
	for val in img0_q:
		f.write(f"{val}")

print(y_test[0:1])
print(f"in_scale: {in_scale}")
print(f"in_zp: {in_zp}")

# 3.  Run the network once
feat_interp.invoke()

# 4.  Locate the first‑convolution output tensor
conv1_details = next(
    t for t in feat_interp.get_tensor_details()
    if t["name"].startswith("sequential_1/conv2d_1/Relu")
)
conv1_q = feat_interp.get_tensor(conv1_details["index"])

# 5.  De‑quantise to float so it’s human‑readable
c_scale, c_zp = conv1_details["quantization"]
conv1_i = conv1_q
conv1_f = (conv1_q.astype(np.float32) - c_zp) * c_scale

# 6.  Inspect a small patch (top‑left 5×5 of channel 0)
#print("\n--- First‑convolution output (de‑quantised) for test image 0 ---")
#print(conv1_i[0,:,:,:])
#print(conv1_f[0,:,:,:])
#print(conv1_f[0, :5, :5, 0])

for filter in range(12):
	quantized_filter = conv1_i[0,:,:,filter]
	full_filter = conv1_f[0,:,:,filter]

	fname = out_dir / f"quantized_{filter}"
	with open(fname, "w") as f:
		for row in quantized_filter:
			for col in row:
				f.write(f"{col},")
			f.write("\n")

	fname = out_dir / f"full_{filter}"

	with open(fname, "w") as f:
		for row in full_filter:
			for col in row:
				f.write(f"{col},")
			f.write("\n")

	fname = out_dir / f"relu_{filter}"
	with open(fname, "w") as f:
		for row in quantized_filter:
			for col in row:
				f.write(f"{0 if col < 1 else col},")
			f.write("\n")

	fname = out_dir / f"full_{filter}"