#ifndef USER_H
#define USER_H

#include <linux/ioctl.h>
#include <stdint.h>

// Define the magic number for the accelerator's ioctls
#define ACCEL_IOC_MAGIC 'q'

// Structure to hold the 10 dense output values (32-bits)
typedef struct {
    uint32_t prob_array[10];
} dense_array10_t;

// Main IOCTL argument structure with a union
typedef struct {
    union {
        uint32_t system_status_val; // For reading status like dense_done
        uint32_t image_data;
        dense_array10_t prob; // prob array for 10 dense classification outputs
    } data;
} accelerator_arg_t;

// --- IOCTL Command Definitions ---
// All commands use accelerator_arg_t

#define ACCEL_LOAD_IMAGE_WORD _IOW(ACCEL_IOC_MAGIC, 1, accelerator_arg_t)
#define ACCEL_DENSE_DONE_SIG _IOR(ACCEL_IOC_MAGIC, 2, accelerator_arg_t)
#define ACCEL_READ_DENSE_OUTPUTS _IOR(ACCEL_IOC_MAGIC, 3, accelerator_arg_t)

/*
#define PIXEL_DATA_MASK     (0x1)       // Bit 0
#define PIXEL_DATA_POS      0

#define ADDRESS_MASK        (0x3FF << 1) // 10 bits (0x3FF) starting at bit 1
#define ADDRESS_POS         1

#define LOAD_DONE_MASK      (0x1 << 11)  // Bit 11
#define LOAD_DONE_POS       11
*/

#endif USER_H